@@ -254,12 +254,14 @@ __rpc_getconfip(nettype)
 		while ((nconf = getnetconfig(confighandle)) != NULL) {
 			if (strcmp(nconf->nc_protofmly, NC_INET) == 0 ||
 			    strcmp(nconf->nc_protofmly, NC_INET6) == 0) {
-				if (strcmp(nconf->nc_proto, NC_TCP) == 0) {
+				if (strcmp(nconf->nc_proto, NC_TCP) == 0 &&
+						netid_tcp == NULL) {
 					netid_tcp = strdup(nconf->nc_netid);
 					thr_setspecific(tcp_key,
 							(void *) netid_tcp);
 				} else
-				if (strcmp(nconf->nc_proto, NC_UDP) == 0) {
+				if (strcmp(nconf->nc_proto, NC_UDP) == 0 &&
+						netid_udp == NULL) {
 					netid_udp = strdup(nconf->nc_netid);
 					thr_setspecific(udp_key,
 						(void *) netid_udp);
@@ -525,11 +527,18 @@ int
 __rpc_nconf2fd(const struct netconfig *nconf)
 {
 	struct __rpc_sockinfo si;
+	int fd;
 
 	if (!__rpc_nconf2sockinfo(nconf, &si))
 		return 0;
 
-	return socket(si.si_af, si.si_socktype, si.si_proto);
+	if ((fd = socket(si.si_af, si.si_socktype, si.si_proto)) >= 0 &&
+	    si.si_af == AF_INET6) {
+		int val = 1;
+
+		setsockopt(fd, SOL_IPV6, IPV6_V6ONLY, &val, sizeof(val));
+	}
+	return fd;
 }
 
 int
@@ -594,6 +603,9 @@ __rpc_taddr2uaddr_af(int af, const struct netbuf *nbuf)
 #endif
 	u_int16_t port;
 
+	if (nbuf->len <= 0)
+		return NULL;
+
 	switch (af) {
 	case AF_INET:
 		sin = nbuf->buf;
@@ -622,7 +634,7 @@ __rpc_taddr2uaddr_af(int af, const struct netbuf *nbuf)
 		/*	if (asprintf(&ret, "%.*s", (int)(sun->sun_len -
 		    offsetof(struct sockaddr_un, sun_path)),
 		    sun->sun_path) < 0)*/
-		if (asprintf(&ret, "%.*s", (int)(sizeof(sun) -
+		if (asprintf(&ret, "%.*s", (int)(sizeof(*sun) -
 						 offsetof(struct sockaddr_un, sun_path)),
 			     sun->sun_path) < 0)
 
@@ -826,3 +838,22 @@ __rpc_sockisbound(int fd)
 
 	return 0;
 }
+
+/*
+ * Helper function to set up a netbuf
+ */
+struct netbuf *
+__rpc_set_netbuf(struct netbuf *nb, const void *ptr, size_t len)
+{
+	if (nb->len != len) {
+		if (nb->len)
+			mem_free(nb->buf, nb->len);
+		nb->buf = mem_alloc(len);
+		if (nb->buf == NULL)
+			return NULL;
+
+		nb->maxlen = nb->len = len;
+	}
+	memcpy(nb->buf, ptr, len);
+	return nb;
+}
