@@ -103,17 +103,27 @@ struct cf_conn {  /* kept in xprt->xp_p1 for actual connection */
 	struct timeval last_recv_time;
 };
 
-static void
-map_ipv4_to_ipv6(sin, sin6)
-	struct sockaddr_in *sin;
-	struct sockaddr_in6 *sin6;
+/*
+ * This is used to set xprt->xp_raddr in a way legacy
+ * apps can deal with
+ */
+void
+__xprt_set_raddr(SVCXPRT *xprt, const struct sockaddr_storage *ss)
 {
-	sin6->sin6_family = AF_INET6;
-	sin6->sin6_port = sin->sin_port;
-	sin6->sin6_addr.s6_addr32[0] = 0;
-	sin6->sin6_addr.s6_addr32[1] = 0;
-	sin6->sin6_addr.s6_addr32[2] = htonl(0xffff);
-	sin6->sin6_addr.s6_addr32[3] = *(uint32_t *)&sin->sin_addr;
+	switch (ss->ss_family) {
+	case AF_INET6:
+		memcpy(&xprt->xp_raddr, ss, sizeof(struct sockaddr_in6));
+		xprt->xp_addrlen = sizeof (struct sockaddr_in6);
+		break;
+	case AF_INET:
+		memcpy(&xprt->xp_raddr, ss, sizeof(struct sockaddr_in));
+		xprt->xp_addrlen = sizeof (struct sockaddr_in);
+		break;
+	default:
+		xprt->xp_raddr.sin6_family = AF_UNSPEC;
+		xprt->xp_addrlen = sizeof (struct sockaddr);
+		break;
+	}
 }
 
 /*
@@ -174,14 +184,10 @@ svc_vc_create(fd, sendsize, recvsize)
 		goto cleanup_svc_vc_create;
 	}
 
-	xprt->xp_ltaddr.maxlen = xprt->xp_ltaddr.len = sizeof(sslocal);
-	xprt->xp_ltaddr.buf = mem_alloc((size_t)sizeof(sslocal));
-	if (xprt->xp_ltaddr.buf == NULL) {
+	if (!__rpc_set_netbuf(&xprt->xp_ltaddr, &sslocal, sizeof(sslocal))) {
 		warnx("svc_vc_create: no mem for local addr");
 		goto cleanup_svc_vc_create;
 	}
-	memcpy(xprt->xp_ltaddr.buf, &sslocal, (size_t)sizeof(sslocal));
-	xprt->xp_rtaddr.maxlen = sizeof (struct sockaddr_storage);
 	xprt_register(xprt);
 	return (xprt);
 cleanup_svc_vc_create:
@@ -201,7 +207,6 @@ svc_fd_create(fd, sendsize, recvsize)
 	u_int recvsize;
 {
 	struct sockaddr_storage ss;
-	struct sockaddr_in6 sin6;
 	socklen_t slen;
 	SVCXPRT *ret;
 
@@ -216,37 +221,23 @@ svc_fd_create(fd, sendsize, recvsize)
 		warnx("svc_fd_create: could not retrieve local addr");
 		goto freedata;
 	}
-	ret->xp_ltaddr.maxlen = ret->xp_ltaddr.len = sizeof(ss);
-	ret->xp_ltaddr.buf = mem_alloc((size_t)sizeof(ss));
-	if (ret->xp_ltaddr.buf == NULL) {
+	if (!__rpc_set_netbuf(&ret->xp_ltaddr, &ss, sizeof(ss))) {
 		warnx("svc_fd_create: no mem for local addr");
 		goto freedata;
 	}
-	memcpy(ret->xp_ltaddr.buf, &ss, (size_t)sizeof(ss));
+
 	slen = sizeof (struct sockaddr_storage);
 	if (getpeername(fd, (struct sockaddr *)(void *)&ss, &slen) < 0) {
 		warnx("svc_fd_create: could not retrieve remote addr");
 		goto freedata;
 	}
-	if (ss.ss_family == AF_INET) {
-		map_ipv4_to_ipv6((struct sockaddr_in *)&ss, &sin6);
-	} else {
-		memcpy(&sin6, &ss, sizeof(ss));
-	}
-	ret->xp_rtaddr.maxlen = ret->xp_rtaddr.len = sizeof(ss);
-	ret->xp_rtaddr.buf = mem_alloc((size_t)sizeof(ss));
-	if (ret->xp_rtaddr.buf == NULL) {
+	if (!__rpc_set_netbuf(&ret->xp_rtaddr, &ss, sizeof(ss))) {
 		warnx("svc_fd_create: no mem for local addr");
 		goto freedata;
 	}
-	memcpy(ret->xp_rtaddr.buf, &sin6, (size_t)sizeof(ss));
-#ifdef PORTMAP
-	if (sin6.sin6_family == AF_INET6 || sin6.sin6_family == AF_LOCAL) {
-		memcpy(&ret->xp_raddr, ret->xp_rtaddr.buf,
-			sizeof(struct sockaddr_in6));
-		ret->xp_addrlen = sizeof (struct sockaddr_in6);
-	}
-#endif				/* PORTMAP */
+
+	/* Set xp_raddr for compatibility */
+	__xprt_set_raddr(ret, &ss);
 
 	return ret;
 
@@ -309,7 +300,6 @@ rendezvous_request(xprt, msg)
 	struct cf_rendezvous *r;
 	struct cf_conn *cd;
 	struct sockaddr_storage addr;
-	struct sockaddr_in6 sin6;
 	socklen_t len;
 	struct __rpc_sockinfo si;
 	SVCXPRT *newxprt;
@@ -341,24 +331,12 @@ again:
 	 */
 
 	newxprt = makefd_xprt(sock, r->sendsize, r->recvsize);
-	if (addr.ss_family == AF_INET) {
-		map_ipv4_to_ipv6((struct sockaddr_in *)&addr, &sin6);
-		len = sizeof(struct sockaddr_in6);
-	} else {
-		memcpy(&sin6, &addr, len);
-	}
-	newxprt->xp_rtaddr.buf = mem_alloc(len);
-	if (newxprt->xp_rtaddr.buf == NULL)
+
+	if (!__rpc_set_netbuf(&newxprt->xp_rtaddr, &addr, len))
 		return (FALSE);
-	memcpy(newxprt->xp_rtaddr.buf, &sin6, len);
-	newxprt->xp_rtaddr.maxlen = newxprt->xp_rtaddr.len = len;
-#ifdef PORTMAP
-	if (sin6.sin6_family == AF_INET6 || sin6.sin6_family == AF_LOCAL) {
-		memcpy(&newxprt->xp_raddr, newxprt->xp_rtaddr.buf,
-			sizeof(struct sockaddr_in6));
-		newxprt->xp_addrlen = len;
-	}
-#endif				/* PORTMAP */
+
+	__xprt_set_raddr(newxprt, &addr);
+
 	if (__rpc_fd2sockinfo(sock, &si) && si.si_proto == IPPROTO_TCP) {
 		len = 1;
 		/* XXX fvdl - is this useful? */
